#!/usr/bin/python3
# -- coding: UTF-8 --

import json
import psutil
import time
import socket
import ctypes
from ctypes import *
from pydbus import SystemBus
from gi.repository import GLib, Gtk, Gio
import gi
gi.require_version('Gtk', '3.0')
import dbus
import dbus.service
import dbus.mainloop.glib
from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)
from kysdk_logsec import kylog
from kdk_accessctl import kyaccessctl
from enum import Enum

BUS_NAME = "com.kylin.kysdk.ListControl"
BUS_PATH = "/com/kylin/kysdk/ListControl"
BUS_IFACE = "com.kylin.kysdk.ListControl"

class KdkDeviceState(Enum):
    MDM_DEV_STATE_DISBALE = 0
    MDM_DEV_STATE_READONLY = 1
    MDM_DEV_STATE_ENABLE = 2

class DevInfo(Structure):
    _fields_ = [
        ('module', c_int),
        ('state', c_int),
        ('pid', c_char * 5),
        ('vid', c_char * 5),
        ('product', c_char * 128),
        ('manufacturer', c_char * 128),
        ('devpath', c_char * 256),
    ]

class ListControl(dbus.service.Object):
    """
    <node>
        <interface name='com.kylin.kysdk.ListControl'>
             <method name='SetWhiteList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetWhiteList' >
                <arg type='as' direction='out' name='white-list'/>
            </method>
            <method name='ClearWhiteList'>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='DelWhiteList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
             <method name='SetBlackList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetBlackList' >
                <arg type='as' direction='out' name='black-list'/>
            </method>
            <method name='ClearBlackList'>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='DelBlackList' >
                <arg type='s' direction='in' name='pid'/>
                <arg type='s' direction='in' name='vid'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='SetWhiteListByBatch'>
                <arg type='a(ss)' direction='in' name='whitelist'/>
                <arg type='i' direction='in' name='count'/>
                <arg type='i' direction ='out' name='ret'/>
            </method>
            <method name='DelWhiteListByBatch'>
                <arg type='a(ss)' direction='in' name='whitelist'/>
                <arg type='i' direction='in' name='count'/>
                <arg type='i' direction ='out' name='ret'/>
            </method>
            <method name='SetBlackListByBatch'>
                <arg type='a(ss)' direction='in' name='blacklist'/>
                <arg type='i' direction='in' name='count'/>
                <arg type='i' direction ='out' name='ret'/>
            </method>
            <method name='DelBlackListByBatch'>
                <arg type='a(ss)' direction='in' name='blacklist'/>
                <arg type='i' direction='in' name='count'/>
                <arg type='i' direction ='out' name='ret'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        bus = dbus.SystemBus()
        bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
        dbus.service.Object.__init__(self, bus_name, BUS_PATH)

        self.so = ctypes.CDLL("/usr/lib/mdm/libdevicesctl.so")

        try:
            self.log = SystemBus().get("org.log.sys_transmit",
                                   "/org/log/sys_transmit")
        except Exception:
            print("log service is error")

    @property
    def get_host_name(self) -> str:
        """获取当前用户名"""
        json_user_list = json.dumps(psutil.users())
        user_list = json.loads(json_user_list)
        if len(user_list) < 1:
            return "root"
        return user_list[0][0]

    @property
    def get_host_time(self) -> str:
        """获取本机时间
        return: time_1
        """
        time_1 = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
        return time_1

    @property
    def get_host_ip(self) -> str:
        """
        查询本机ip地址
        :return: ip
        """
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            ip = s.getsockname()[0]
        except Exception:
            return "unknown"
        return ip

    def log_transmit(self, message: str) -> None:
        log = {
            "time":self.get_host_time,
            "hostname":self.get_host_name,
            "ip":self.get_host_ip,
            "name":"list-control",
            "lv":"info",
            "message":message
        }
        json_log = json.dumps(log, ensure_ascii=False)
        print(json_log)

        try:
            self.log.log_transmit(json_log)
        except Exception:
            print("log message error")


    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="ss", out_signature="i", sender_keyword='sender')
    def SetWhiteList(self, pid: str, vid: str, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call_with_params(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteList", f"[Params] pid:{pid},vid:{vid}")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)

        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("添加白名单设备pid: {}, vid: {}".format(pid, vid))
        devinfo = DevInfo()
        devinfo.pid = pid.encode()
        devinfo.vid = vid.encode()
        ret = self.so.kdk_device_mdm_add_whitelist(byref(devinfo))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteList")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="as", sender_keyword='sender')
    def GetWhiteList(self, sender=None) -> list:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetWhiteList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetWhiteList")
            return []

        white_list = []
        count = c_int(0)
        self.so.kdk_device_mdm_get_whitelist.restype = POINTER(DevInfo)
        try:
            p = self.so.kdk_device_mdm_get_whitelist(byref(count))
            for i in range(0, count.value):
                pidvid = str(p[i].pid.decode()) + str(p[i].vid.decode())
                # print(pidvid)
                white_list.append(pidvid)
        except Exception:
            pass

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetWhiteList")
        return white_list

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="i", sender_keyword='sender')
    def ClearWhiteList(self, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearWhiteList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)

        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearWhiteList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("清空白名单")
        ret = self.so.kdk_device_mdm_clear_whitelist()

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearWhiteList")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="ss", out_signature="i", sender_keyword='sender')
    def DelWhiteList(self, pid: str, vid: str, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call_with_params(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteList", f"[Params] pid:{pid},vid:{vid}")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("删除白名单设备pid: {}, vid: {}".format(pid, vid))
        devinfo = DevInfo()
        devinfo.pid = pid.encode()
        devinfo.vid = vid.encode()
        ret = self.so.kdk_device_mdm_del_whitelist(byref(devinfo))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteList")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="ss", out_signature="i", sender_keyword='sender')
    def SetBlackList(self, pid: str, vid: str, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call_with_params(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackList", f"[Params] pid:{pid},vid:{vid}")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("添加黑名单设备pid: {}, vid: {}".format(pid, vid))
        devinfo = DevInfo()
        devinfo.pid = pid.encode()
        devinfo.vid = vid.encode()
        ret = self.so.kdk_device_mdm_add_blacklist(byref(devinfo))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackList")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="as", sender_keyword='sender')
    def GetBlackList(self, sender=None) -> list:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetBlackList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetBlackList")
            return []

        black_list = []
        count = c_int(0)
        self.so.kdk_device_mdm_get_blacklist.restype = POINTER(DevInfo)
        try:
            p = self.so.kdk_device_mdm_get_blacklist(byref(count))
            for i in range(0, count.value):
                pidvid = str(p[i].pid.decode()) + str(p[i].vid.decode())
                # print(pidvid)
                black_list.append(pidvid)
        except Exception:
            pass

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "GetBlackList")
        return black_list

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="", out_signature="i", sender_keyword='sender')
    def ClearBlackList(self, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearBlackList")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearBlackList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("清空黑名单")

        ret = self.so.kdk_device_mdm_clear_blacklist()

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "ClearBlackList")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="ss", out_signature="i", sender_keyword='sender')
    def DelBlackList(self, pid: str, vid: str, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call_with_params(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackList", f"[Params] pid:{pid},vid:{vid}")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, 
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackList")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("删除黑名单设备pid: {}, vid: {}".format(pid, vid))

        devinfo = DevInfo()
        devinfo.pid = pid.encode()
        devinfo.vid = vid.encode()
        ret = self.so.kdk_device_mdm_del_blacklist(byref(devinfo))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackList")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="a(ss)i",out_signature="i", sender_keyword='sender')
    def SetWhiteListByBatch(self, whitelist, count, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call_with_params(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteListByBatch", f"[Params] whitelist:{whitelist}")

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender,
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteListByBatch")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("批量添加白名单设备 count: {}".format(count))
        size = ctypes.sizeof(DevInfo) * count
        buffer = (ctypes.c_char * size)()
        list = []
        len = 0
        for i in whitelist:
            len = len + 1
            if len > count:
                break
            devinfo = DevInfo()
            devinfo.pid = i[0].encode()
            devinfo.vid = i[1].encode()
            list.append(devinfo)
            # print("devinfo.pid = {}, devinfo.vid = {}".format(devinfo.pid, devinfo.vid))

        for i, item in enumerate(list):
            ctypes.memmove(ctypes.addressof(buffer) + i * ctypes.sizeof(DevInfo), ctypes.byref(item), ctypes.sizeof(DevInfo))
        c_array = ctypes.cast(buffer, ctypes.POINTER(DevInfo))

        ret = self.so.kdk_device_mdm_add_whitelist_by_batch(c_array, ctypes.c_int(count))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetWhiteListByBatch")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="a(ss)i",out_signature="i", sender_keyword='sender')
    def DelWhiteListByBatch(self, whitelist, count, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call_with_params(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteListByBatch", f"[Params] whitelist:{whitelist}")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender,
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteListByBatch")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("批量删除白名单设备 count: {}".format(count))
        size = ctypes.sizeof(DevInfo) * count
        buffer = (ctypes.c_char * size)()
        list = []
        len = 0
        for i in whitelist:
            len = len + 1
            if len > count:
                break
            devinfo = DevInfo()
            devinfo.pid = i[0].encode()
            devinfo.vid = i[1].encode()
            list.append(devinfo)
            # print("devinfo.pid = {}, devinfo.vid = {}".format(devinfo.pid, devinfo.vid))

        for i, item in enumerate(list):
            ctypes.memmove(ctypes.addressof(buffer) + i * ctypes.sizeof(DevInfo), ctypes.byref(item), ctypes.sizeof(DevInfo))
        c_array = ctypes.cast(buffer, ctypes.POINTER(DevInfo))

        ret = self.so.kdk_device_mdm_del_whitelist_by_batch(c_array, ctypes.c_int(count))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelWhiteListByBatch")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="a(ss)i",out_signature="i", sender_keyword='sender')
    def SetBlackListByBatch(self, blacklist, count, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call_with_params(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackListByBatch", f"[Params] blacklist:{blacklist}")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender,
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackListByBatch")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("批量删除黑名单设备 count: {}".format(count))
        size = ctypes.sizeof(DevInfo) * count
        buffer = (ctypes.c_char * size)()
        list = []
        len = 0
        for i in blacklist:
            len = len + 1
            if len > count:
                break
            devinfo = DevInfo()
            devinfo.pid = i[0].encode()
            devinfo.vid = i[1].encode()
            list.append(devinfo)
            # print("devinfo.pid = {}, devinfo.vid = {}".format(devinfo.pid, devinfo.vid))

        for i, item in enumerate(list):
            ctypes.memmove(ctypes.addressof(buffer) + i * ctypes.sizeof(DevInfo), ctypes.byref(item), ctypes.sizeof(DevInfo))
        c_array = ctypes.cast(buffer, ctypes.POINTER(DevInfo))

        ret = self.so.kdk_device_mdm_add_blacklist_by_batch(c_array, ctypes.c_int(count))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "SetBlackListByBatch")
        return ret

    @dbus.service.method(dbus_interface=BUS_IFACE, in_signature="a(ss)i",out_signature="i", sender_keyword='sender')
    def DelBlackListByBatch(self, blacklist, count, sender=None) -> int:
        kylog.KdkSecLog().kdklog_dbus_call_with_params(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackListByBatch", f"[Params] blacklist:{blacklist}")
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender,
        kyaccessctl.KdkModuleType.KDK_MODULE_DESKTOP.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackListByBatch")
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        self.log_transmit("批量删除黑名单设备 count: {}".format(count))
        size = ctypes.sizeof(DevInfo) * count
        buffer = (ctypes.c_char * size)()
        list = []
        len = 0
        for i in blacklist:
            len = len + 1
            if len > count:
                break
            devinfo = DevInfo()
            devinfo.pid = i[0].encode()
            devinfo.vid = i[1].encode()
            list.append(devinfo)
            # print("devinfo.pid = {}, devinfo.vid = {}".format(devinfo.pid, devinfo.vid))

        for i, item in enumerate(list):
            ctypes.memmove(ctypes.addressof(buffer) + i * ctypes.sizeof(DevInfo), ctypes.byref(item), ctypes.sizeof(DevInfo))
        c_array = ctypes.cast(buffer, ctypes.POINTER(DevInfo))

        ret = self.so.kdk_device_mdm_del_blacklist_by_batch(c_array, ctypes.c_int(count))

        kylog.KdkSecLog().kdklog_dbus_exit(sender, kylog.KdkLogModule.KDK_MODULE_DESKTOP.value, BUS_NAME, BUS_PATH, BUS_IFACE, "DelBlackListByBatch")
        return ret

if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = dbus.SystemBus()
    bus.request_name(BUS_NAME)
    ListControl()
    loop.run()
